package de.unibielefeld.cebitec.gi.algogr.exl2.misc;
import java.util.Random;

/**
 * Implementation of M. Vose's improved alias method. The algorithm provides
 * sampling of arbitrary distributions over finite space.
 * 
 * M. Vose: A Linear Algorithm For Generating Random Numbers With a Given
 * Distribution
 * 
 * Implemented by MAT???AS GIOVANNINI under Creative Commons License CC0 1.0
 * Universal (CC0 1.0)
 */
public class AliasMethod {

	private final Random random;
	private final int limit;
	private final double[] prob;
	private final int[] alias;

	public AliasMethod(final double[] pa) {
		this(pa, new Random());
	}

	public int getLimit() {
		return limit;
	}

	public AliasMethod(final double[] pa, final Random random) {
		final int limit = pa.length;
		if (limit == 0)
			throw new IllegalArgumentException("Vose");
		double sum = 0;
		for (int j = 0; j != limit; j++) {
			if (pa[j] < 0)
				throw new IllegalArgumentException("Vose");
			sum += pa[j];
		}
		final double scale = (double) limit / sum;
		final double[] sa = new double[limit];
		for (int j = 0; j != limit; j++)
			sa[j] = pa[j] * scale;

		this.random = random;
		this.limit = limit;
		this.prob = new double[limit];
		this.alias = new int[limit];
		init(sa);
	}

	private void init(final double[] sa) {
		final int[] small = new int[limit];
		final int[] large = new int[limit];
		int ns = 0;
		int nl = 0;
		for (int j = 0; j != limit; j++)
			if (sa[j] > 1)
				large[nl++] = j;
			else
				small[ns++] = j;
		while (ns != 0 && nl != 0) {
			final int j = small[--ns];
			final int k = large[--nl];
			prob[j] = sa[j];
			alias[j] = k;
			sa[k] = (sa[k] + sa[j]) - 1; // sic
			if (sa[k] > 1)
				large[nl++] = k;
			else
				small[ns++] = k;
		}
		while (ns != 0)
			prob[small[--ns]] = 1;
		while (nl != 0)
			prob[large[--nl]] = 1;
	}

	public int choose() {
		final double u = limit * random.nextDouble();
		final int j = (int) Math.floor(u);
		final double p = u - (double) j;
		return p <= prob[j] ? j : alias[j];
	}
}